@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -51,7 +51,7 @@ def _dedup_scopes(scopes):
         max_parent_path = tuple(reversed(path))
       path.append(scope.name)
       scope = scope.parent
-    if max_parent is not leaf:
+    if max_parent is not leaf and leaf in minimal_set:
       del minimal_set[leaf]
     paths.append((max_parent, max_parent_path))
   return tuple(minimal_set), tuple(paths)
@@ -94,7 +94,7 @@ def pack(fn: Callable[..., Any],
           scope._variables, in_variable_filters))
     variable_groups_xs_t = _transpose(variable_groups_xs)
 
-    # Make sure in only variable collections are frozen
+    # Make sure that in-only variable collections are frozen
     for variable_group_xs in variable_groups_xs_t:
       for variable_group in variable_group_xs:
         for col_name, collection in variable_group.items():
@@ -209,9 +209,9 @@ def transform(
     trans_in_fn: creates a view of the target variables.
     trans_out_fn: transforms the updated variables in the view after mutation.
     init: If True, variables are initialized before transformation.
-    rngs: PRNGSequences added tot the transformed scope (default: all).
+    rngs: PRNGSequences added to the transformed scope (default: all).
     variables: Addtional Variable collections added to the transformed scope.
-      Beside those specified by `target` (default: all).
+      Besides those specified by `target` (default: all).
   """
   def wrapper(scope_fn, repack, variable_groups, rng_groups, treedef, *args):
     target, variables = variable_groups
@@ -252,7 +252,7 @@ def transform_module(fn: Callable[..., Any],
                      mutable: bool = False,
                      rngs: PRNGSequenceFilter = True,
                      variables: CollectionFilter = True):
-  """"Wrapper around `tranform` for automatic init detection.
+  """"Wrapper around `transform` for automatic init detection.
 
   This function will detect if the target collection exists.
   If it doesn't `init=True` is will be passed to `transform`.
@@ -310,14 +310,14 @@ InOutAxis = Union[Axis, In[Axis], Out[Axis]]
 def vmap(fn: Callable[..., Any],
          variable_axes: Mapping[CollectionFilter, InOutAxis],
          split_rngs: Mapping[PRNGSequenceFilter, bool],
-         in_axes=0, out_axes=0, axis_size=None) -> Callable[..., Any]:
+         in_axes=0, out_axes=0, axis_size=None, axis_name=None) -> Callable[..., Any]:
   """A lifted version of `jax.vmap`.
 
   See `jax.vmap` for the unlifted batch transform in Jax.
 
   Example::
 
-    # a dense mapping with seperate parameters for axis 0.
+    # a dense mapping with separate parameters for axis 0.
     batch_dense = lift.vmap(
         nn.dense,
         in_axes=(0, None),
@@ -333,8 +333,11 @@ def vmap(fn: Callable[..., Any],
       of the batch dimension. Unsplit PRNGs will be broadcasted.
     in_axes: Specifies the mapping of the input arguments (see `jax.vmap).
     out_axes: Specifies the mapping of the return value (see `jax.vmap).
-    axes_size: Specifies the size of the batch axis. This only needs
-      to be specified if it cannot be derivied from the input arguments.
+    axis_size: Specifies the size of the batch axis. This only needs
+      to be specified if it cannot be derived from the input arguments.
+    axis_name: Specifies a name for the batch axis. Can be used together
+      with parallel reduction primitives (e.g. `jax.lax.pmean`,
+      `jax.lax.ppermute`, etc.)
   """
   variable_in_axes, variable_out_axes = _split_in_out_axes(variable_axes)
   variable_in_groups, variable_in_axes = _unzip2(variable_in_axes.items())
@@ -369,7 +372,8 @@ def vmap(fn: Callable[..., Any],
 
     @functools.partial(jax.vmap,
                        in_axes=(variable_in_axes, rng_axes, in_axes),
-                       out_axes=(out_axes, variable_out_axes))
+                       out_axes=(out_axes, variable_out_axes),
+                       axis_name=axis_name)
     @functools.wraps(fn)
     def mapped(variable_groups, rng_groups, args):
       scope = scope_fn(variable_groups, rng_groups)
@@ -416,17 +420,18 @@ def scan(fn: Callable[..., Any],
 
   Args:
     fn: the function to be transformed.
-    variable_axes: the variable collections that are scannned over.
+    variable_axes: the variable collections that are scanned over.
     variable_broadcast: Specifies the broadcasted variable collections.
       A broadcasted variable should not depend on any computation that cannot be lifted out of the loop.
-      This is typically used to define shared paramaters inside the fn.
+      This is typically used to define shared parameters inside the fn.
     variable_carry: Specifies the variable collections that are carried through the loop.
       Mutations to these variables are carried to the next iteration and will be preserved
       when the scan finishes.
     split_rngs: Split PRNG sequences will be different for each loop iterations.
       If split is False the PRNGs will be the same across iterations.
     in_axes: Specifies the axis to scan over for the arguments. Should be a prefix
-      tree of the arguments.
+      tree of the arguments. Use `flax.core.broadcast` to feed an entire input
+      to each iteration of the scan body.
     out_axes: Specifies the axis to scan over for the return value. Should be a prefix
       tree of the return value.
     length: Specifies the number of loop iterations. This only needs
@@ -446,7 +451,7 @@ def scan(fn: Callable[..., Any],
             variable_groups, rng_groups,
             init, *args):
     def find_length(axis, x):
-      if axis is not None:
+      if axis is not axes_scan.broadcast:
         leaves = jax.tree_leaves(x)
         if leaves:
           return leaves[0].shape[axis]
@@ -535,10 +540,10 @@ def custom_vjp(fn: Callable[..., Any], backward_fn: Callable[..., Any],
     dense_sign_grad = lift.custom_vjp(fwd, backward_fn=bwd, nondiff_argnums=(2,))
 
   Args:
-    fn: should return a tuple of output and auxilairy data for the backward pass.
+    fn: should return a tuple of output and auxilliary data for the backward pass.
     backward_fn: arguments are passed as (*nondiff_args, scope_fn, grad_variables, aux, g_y)
-      where scope_fn takes grad_varialbes to create the scope,
-      aux is the auxilairy data returend by `fn`,
+      where scope_fn takes grad_variables to create the scope,
+      aux is the auxilliary data returend by `fn`,
       and g_y is the tangent of y.
   """
   # TODO(jheek) is this transform general/flexible enough?
