@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,38 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Copyright 2020 The Flax Authors.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 """Flax functional core: Scopes."""
 
 import contextlib
 import functools
 import hashlib
-from typing import Any, Callable, Container, Dict, Set, Iterable, Optional, Sequence, Tuple, TypeVar, Union, Generic, Mapping
+from typing import Any, Callable, Container, Dict, Generic, Iterable, Mapping, Optional, Sequence, Set, Tuple, TypeVar, Union
 
 from . import tracers
+from flax.errors import InvalidFilterError, InvalidScopeError, NameInUseError, NameTypeError, VariableModificationError
 from .frozen_dict import freeze
 from .frozen_dict import FrozenDict
 from .frozen_dict import unfreeze
-
 import jax
-from jax import random
 from jax import numpy as jnp
+from jax import random
 
 T = TypeVar('T')
 
-
 PRNGKey = Any
 Array = Any
 
@@ -57,12 +43,13 @@ Collection = Mapping[str, Any]
 MutableCollection = Dict[str, Any]
 
 VariableDict = Mapping[str, Collection]
+FrozenVariableDict = FrozenDict[str, Collection]
 MutableVariableDict = Dict[str, MutableCollection]
 
 
 def _fold_in_str(rng: PRNGKey, data: str) -> PRNGKey:
   """Folds a string into a jax.random.PRNGKey using its SHA-1 hash.
-  
+
   This is faster than splitting an PRNGKey because it allows generating new PRNG
   keys in parellel that are independent of each other.
 
@@ -82,12 +69,12 @@ def _fold_in_str(rng: PRNGKey, data: str) -> PRNGKey:
 
 def in_filter(filter_like: Filter, col: str) -> bool:
   """Checks whether a filter can be applied to a collection.
-  
+
   Used for both collections and rng sequence filters.
 
   Args:
-    filter_like: a filter (either a boolean, a string, or a list of strings)
-      for a collection.
+    filter_like: a filter (either a boolean, a string, or a list of strings) for
+      a collection.
     col: a collection, which is a string identifying a dictionary of data, for
       instance "params" or "batch_stats".
 
@@ -101,15 +88,15 @@ def in_filter(filter_like: Filter, col: str) -> bool:
     return col in filter_like
   if isinstance(filter_like, bool):
     return filter_like
-  raise TypeError(f'Invalid Filter: "{filter_like}"')
+  raise InvalidFilterError(filter_like)
 
 
 def filter_to_set(x: Filter) -> Set[str]:
   """Converts a Filter into a set of collections, fails on the infinite set.
-  
+
   Args:
     x: a filter (boolean, string, or list of strings).
-    
+
   Returns:
     The input filter represented as a set of strings.
   """
@@ -120,18 +107,18 @@ def filter_to_set(x: Filter) -> Set[str]:
     return set([x])
   if isinstance(x, Iterable):
     return set(x)
-  raise TypeError('Invalid Filter')
+  raise InvalidFilterError(x)
 
 
 def union_filters(a: Filter, b: Filter) -> Filter:
   """Takes the union of two filters (similar to a logical or).
-  
-  Arguments:
+
+  Args:
     a: a filter.
     b: a filter.
-    
+
   Returns:
-    The union of the two input filters. For instance, 
+    The union of the two input filters. For instance,
     `union_filters('f1', ['f2']) = {'f1', 'f2'}`.
   """
   if a is True or b is True:
@@ -143,13 +130,13 @@ def union_filters(a: Filter, b: Filter) -> Filter:
 
 def intersect_filters(a: Filter, b: Filter) -> Filter:
   """Take the intersection of two filters (similar to a logical and).
-  
-  Arguments:
+
+  Args:
     a: a filter.
     b: a filter.
-    
+
   Returns:
-    The intersection of the two input filters. For instance, 
+    The intersection of the two input filters. For instance,
     `intersect_filters('f1', ['f1', 'f2']) = {'f1'}`.
   """
   if a is True:
@@ -161,18 +148,19 @@ def intersect_filters(a: Filter, b: Filter) -> Filter:
   return a.intersection(b)
 
 
-def group_collections(xs: VariableDict,
-                col_filters: Sequence[CollectionFilter]) -> Sequence[MutableVariableDict]:
+def group_collections(
+    xs: VariableDict,
+    col_filters: Sequence[CollectionFilter]) -> Sequence[MutableVariableDict]:
   """Groups variables by collection filters.
 
   Iteratively applies the filters in `col_filters` to `xs`, and adds the result
   of applying each filter to the output sequence. Each key in `xs` is only added
   to the output once.
-  
+
   Args:
     xs: a dictionary of variables, keyed by collections (strings).
     col_filters: a list of collection filters.
-    
+
   Returns:
     A sequence S with `len(S) == len(col_filters)`. Each `S[i]` is the result of
     applying filter `col_filters[i]` to the remaining keys in `xs`.
@@ -193,6 +181,8 @@ def group_collections(xs: VariableDict,
 
 
 T = TypeVar('T')
+
+
 class Variable(Generic[T]):
   """A Variable object allows mutable access to a variable in a VariableDict.
 
@@ -223,15 +213,20 @@ class Variable(Generic[T]):
     """Updates the value of this Variable."""
     self.scope.put_variable(self.collection, self.name, value)
 
+  def is_mutable(self) -> bool:
+    """Checks if this Variable is mutable."""
+    return self.scope.is_mutable_collection(self.collection)
+
 
 class Scope:
   """A Scope allows easy access to variables and manages RNGS of a neural network layer.
-  
-  Scopes are purely functional and encapsulated in 
-  :class:`flax.linen.module.Module`, so users writing neural network code 
+
+  Scopes are purely functional and encapsulated in
+  :class:`flax.linen.module.Module`, so users writing neural network code
   usually generally do not interact with ``Scopes`` directly.
 
-  See `core design tests <https://github.com/google/flax/tree/master/tests/core/design>`_
+  See `core design tests
+  <https://github.com/google/flax/tree/master/tests/core/design>`_
   for a number of examples using ``Scopes``.
   """
 
@@ -279,7 +274,7 @@ class Scope:
 
   def _check_valid(self):
     if self._invalid:
-      raise ValueError('This scope is no longer valid.')
+      raise InvalidScopeError(self.name)
 
   @contextlib.contextmanager
   def temporary(self):
@@ -306,33 +301,36 @@ class Scope:
 
     Args:
       rewind_rngs: if true, reset the RNG counter of this scope.
+
     Returns:
-      A rewound version of this scope, which means reservations and children are 
+      A rewound version of this scope, which means reservations and children are
       emptied, and the rng counter is optionally rewound.
     """
     self._check_valid()
-    scope = Scope(self._variables, self.rngs, self.name, self.mutable, self.parent)
+    scope = Scope(self._variables, self.rngs, self.name, self.mutable,
+                  self.parent)
     if not rewind_rngs:
       scope.rng_counters = self.rng_counters
     return scope
 
   def reserve(self, name: str):
     """Reserves a name for a child Scope or Variable.
-    
+
     Args:
       name: the name to reserve.
     """
     if not isinstance(name, str):
-      raise ValueError('Variable and child scopes should have a string name.')
+      raise NameTypeError(name)
     if name in self.reservations:
-      raise ValueError(f'Duplicate use of name: "{name}"')
+      raise NameInUseError(name)
     self.reservations.add(name)
 
   def default_name(self, prefix: str) -> str:
     """Generates an unreserved name with the given prefix.
-    
+
     Args:
       prefix: prefix to use for generating an unreserved name.
+
     Returns:
       The generated name.
     """
@@ -343,14 +341,18 @@ class Scope:
         return name
       i += 1
 
-  def push(self, name: Optional[str] = None, prefix: str = '', reuse=False) -> 'Scope':
+  def push(self,
+           name: Optional[str] = None,
+           prefix: str = '',
+           reuse=False) -> 'Scope':
     """Creates a child Scope.
-    
+
     Args:
       name: optional name of the child.
       prefix: prefix used for generating the name if `name` is `None`.
       reuse: if True will return a pre-existing child scope with the given name
         instead of throwing an error.
+
     Returns:
       The child scope.
     """
@@ -362,7 +364,11 @@ class Scope:
       return self._children[name]
     self.reserve(name)
     rngs = {key: _fold_in_str(rng, name) for key, rng in self.rngs.items()}
-    scope = Scope({}, name=name, rngs=rngs, parent=self, path=self.path + (name,))
+    scope = Scope({},
+                  name=name,
+                  rngs=rngs,
+                  parent=self,
+                  path=self.path + (name,))
     self._children[name] = scope
     return scope
 
@@ -375,14 +381,15 @@ class Scope:
     """Partially applies a child scope to fn.
 
     When calling the returned function multiple times variables will be reused.
-    
+
     Args:
       fn: the function to partially apply the child Scope to.
       name: optional name of the child.
       prefix: prefix used for generating name if it is `None`.
-      named_call: if true, `fn` will be wrapped with `lift.named_call`.
-        The XLA profiler will use this to name tag the computation.
+      named_call: if true, `fn` will be wrapped with `lift.named_call`. The XLA
+        profiler will use this to name tag the computation.
       **partial_kwargs: additional kwargs partially applied to `fn`.
+
     Returns:
       The function with a partially applied scope.
     """
@@ -395,10 +402,12 @@ class Scope:
       # We import named_call at runtime to avoid a circular import issue.
       from . import lift  # type: ignore
       fn = lift.named_call(fn, name)
+
     @functools.wraps(fn)
     def wrapper(*args, **kwargs):
       kwargs = dict(partial_kwargs, **kwargs)
       return fn(scope.rewound(), *args, **kwargs)
+
     return wrapper
 
   def is_mutable_collection(self, col: str) -> bool:
@@ -407,8 +416,7 @@ class Scope:
 
   def _mutable_collection(self, col: str) -> MutableCollection:
     """Returns the collection `col` as a mutable object."""
-    if not self.is_mutable_collection(col):
-      raise ValueError(f'Collection is not mutable: "{col}"')
+    assert self.is_mutable_collection(col), f'Collection {col} is not mutable'
     if col not in self._variables:
       if self.parent:
         parent_col = self.parent._mutable_collection(col)
@@ -445,7 +453,7 @@ class Scope:
 
   def get_variable(self, col: str, name: str, default: T = None) -> T:
     """Retrieves the value of a Variable.
-    
+
     Args:
       col: the variable collection.
       name: the name of the variable.
@@ -464,60 +472,62 @@ class Scope:
 
   def has_variable(self, col: str, name: str) -> bool:
     """Returns true if the given variable exists in this scope.
-    
+
     Args:
       col: the collection of the variable.
-      name: the name of the varaible.
+      name: the name of the variable.
     """
     variables = self._collection(col)
     return name in variables
 
   def put_variable(self, col: str, name: str, value: Any):
-    """Updates the value of the given variable if it is mutable, or an error otherwise. 
-    
+    """Updates the value of the given variable if it is mutable, or an error otherwise.
+
     Args:
       col: the collection of the variable.
       name: the name of the variable.
-      value: the new value of the given variabele.
+      value: the new value of the given variable.
     """
     self._check_valid()
     self._validate_trace_level()
     if not self.is_mutable_collection(col):
-      raise ValueError(
-        f'Trying to update variable "{name}" in "{self.path_text}" '
-        f'but collection "{col}" is immutable.')
+      raise VariableModificationError(col, name, self.path_text)
     variables = self._mutable_collection(col)
     variables[name] = value
 
   def variable(self, col: str, name: str, init_fn: Callable[..., T],
                *init_args) -> Variable[T]:
     """Creates a variable if it doesn't exist yet in this scope and returns it.
-    
+
     Args:
       col: the collection of the variable.
       name: the name of the variable.
-      init_fn: a function taking a PRNGKey plus any other number of
-        positional arguments.
+      init_fn: a function taking a PRNGKey plus any other number of positional
+        arguments.
       *init_args: the arguments to evaluate init_fn on lazily.
+
     Returns:
       The variable.
     """
     self.reserve(name)
     if not self.has_variable(col, name):
-      if not self.is_mutable_collection('params'):
-        raise ValueError(f'No paramater named "{name}" exists in "{self.path_text}".')
+      if not self.is_mutable_collection(col):
+        raise ValueError(
+            f'No Variable named "{name}" for collection "{col}" exists in "{self.path_text}".'
+        )
       init_value = init_fn(*init_args)
       self.put_variable(col, name, init_value)
     return Variable(self, col, name)
 
   def param(self, name: str, init_fn: Callable[..., T], *init_args) -> T:
     """Creates a parameter if it doesn't exist yet in this scope and returns it.
-    
+
     Args:
       name: the name of the parameter.
-      init_fn: a function taking a PRNGKey plus any other number of
-        positional arguments.
+      init_fn: a function taking a PRNGKey plus any other number of positional
+        arguments.
       *init_args: the arguments to evaluate init_fn on lazily.
+
     Returns:
       The parameters.
     """
@@ -534,16 +544,19 @@ class Scope:
       abs_value_flat = jax.tree_leaves(abs_value)
       value_flat = jax.tree_leaves(value)
       for val, abs_val in zip(value_flat, abs_value_flat):
-        # NOTE: We could check dtype consistency here as well but it's 
+        # NOTE: We could check dtype consistency here as well but it's
         # usefuleness is less obvious. We might intentionally change the dtype
         # for inference to a half float type for example.
         if jnp.shape(val) != jnp.shape(abs_val):
-          raise ValueError('Inconsistent shapes between value and initializer '
-                           f'for parameter "{name}" in "{self.path_text}": {jnp.shape(val)}, {jnp.shape(abs_val)}')
+          raise ValueError(
+              'Inconsistent shapes between value and initializer '
+              f'for parameter "{name}" in "{self.path_text}": {jnp.shape(val)}, {jnp.shape(abs_val)}'
+          )
       return value
     else:
       if not self.is_mutable_collection('params'):
-        raise ValueError(f'No paramater named "{name}" exists in "{self.path_text}".')
+        raise ValueError(
+            f'No parameter named "{name}" exists in "{self.path_text}".')
       value = init_fn(self.make_rng('params'), *init_args)
       self.put_variable('params', name, value)
       return value
@@ -553,6 +566,7 @@ class Scope:
     for col in collections:
       self._collection(col)
 
+
 def _unfreeze_variables(variables, mutable):
   new_variables = {}
   for key, value in variables.items():
@@ -570,58 +584,66 @@ def apply(fn: Callable[..., Any],
   Args:
     fn: a function taking a `Scope` as its first argument.
     mutable: the filter determining which variable collections are mutable.
+
   Returns:
     `fn` with the scope partially applied.
   """
+
   @functools.wraps(fn)
-  def wrapper(variables: VariableDict, *args,
-              rngs: Optional[RNGSequences] = None, 
+  def wrapper(variables: VariableDict,
+              *args,
+              rngs: Optional[RNGSequences] = None,
               **kwargs) -> Union[Any, Tuple[Any, VariableDict]]:
-    
+
     if not _is_valid_variables(variables):
       raise ValueError('The first argument passed to an apply function '
                        'should be a dictionary of collections. '
-                       'Each collection should be a `FrozenDict` with string '
-                       'keys.')
+                       'Each collection should be a dictionary '
+                       'with string keys.')
     if rngs is not None and not _is_valid_rngs(rngs):
       raise ValueError('rngs should be a dictionary mapping strings to '
                        '`jax.PRNGKey`.')
     new_variables = _unfreeze_variables(variables, mutable)
     with Scope(new_variables, rngs=rngs, mutable=mutable).temporary() as root:
       y = fn(root, *args, **kwargs)
-    if mutable:
+    if mutable is not False:
       mutated_variables = {k: v
                            for k, v in new_variables.items()
                            if in_filter(mutable, k)}
       return y, freeze(mutated_variables)
     else:
       return y
+
   return wrapper
 
 
-def init(fn: Callable[..., Any], 
-        mutable: CollectionFilter = True) -> Callable[..., Any]:
+def init(fn: Callable[..., Any],
+         mutable: CollectionFilter = True) -> Callable[..., Any]:
   """Functionalize a `Scope` function for initialization.
 
   Args:
     fn: a function taking a `Scope` as its first argument.
     mutable: the filter determining which variable collections are mutable.
+
   Returns:
     `fn` with the scope partially applied.
   """
+
   @functools.wraps(fn)
   def wrapper(rngs, *args, **kwargs) -> Tuple[Any, VariableDict]:
     if not _is_valid_rng(rngs) and not _is_valid_rngs(rngs):
-      raise ValueError('First argument passed to an init function should be a `jax.PRNGKey` '
-                       'or a dictionary mapping strings to `jax.PRNGKey`.') 
+      raise ValueError(
+          'First argument passed to an init function should be a `jax.PRNGKey` '
+          'or a dictionary mapping strings to `jax.PRNGKey`.')
     if not isinstance(rngs, dict):
       rngs = {'params': rngs}
     return apply(fn, mutable=mutable)({}, *args, rngs=rngs, **kwargs)
+
   return wrapper
 
 
 def _is_valid_collection(col: VariableDict):
-  if not isinstance(col, FrozenDict):
+  if not isinstance(col, (FrozenDict, dict)):
     return False
   for name in col.keys():
     # Any value can be stored in a collection so only keys can be verified.
@@ -635,6 +657,7 @@ def _is_valid_variables(variables: VariableDict) -> bool:
 
   Args:
     variables: A variable dict.
+
   Returns:
     True if `variables` is a valid variable dict.
   """
