@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +14,6 @@
 
 """Attention core modules for Flax."""
 
-from collections.abc import Iterable  # pylint: disable=g-importing-member
-
 from functools import partial
 from typing import (Any, Callable, Tuple, Optional)
 
@@ -27,7 +25,7 @@ import numpy as np
 
 from flax.linen.linear import default_kernel_init
 from flax.linen.linear import DenseGeneral
-from flax.linen.module import Module, compact
+from flax.linen.module import Module, compact, merge_param
 from flax.linen.initializers import zeros
 
 PRNGKey = Any
@@ -62,7 +60,7 @@ def dot_product_attention(query: Array,
     value: values to be used in attention with shape of
       `[batch..., kv_length, num_heads, v_depth_per_head]`.
     bias: bias for the attention weights. This should be broadcastable to the
-      shape: `[batch..., num_heads, q_length, kv_length]`
+      shape `[batch..., num_heads, q_length, kv_length]`.
       This can be used for incorporating causal masks, padding masks,
       proximity bias, etc.
     broadcast_dropout: bool: use a broadcasted dropout along batch dims.
@@ -76,13 +74,13 @@ def dot_product_attention(query: Array,
   Returns:
     Output of shape `[batch..., length, num_heads, v_depth_per_head]`.
   """
-  assert key.ndim == query.ndim == value.ndim, "q, k, v must have same rank."
+  assert key.ndim == query.ndim == value.ndim, 'q, k, v must have same rank.'
   assert query.shape[:-3] == key.shape[:-3] == value.shape[:-3], (
-      "q, k, v batch dims must match.")
+      'q, k, v batch dims must match.')
   assert query.shape[-2] == key.shape[-2] == value.shape[-2], (
-      "q, k, v num_heads must match.")
-  assert key.shape[-3] == value.shape[-3], "k, v lengths must match."
-  assert query.shape[-1] == key.shape[-1], "q, k depths must match."
+      'q, k, v num_heads must match.')
+  assert key.shape[-3] == value.shape[-3], 'k, v lengths must match.'
+  assert query.shape[-1] == key.shape[-1], 'q, k depths must match.'
 
   # calculate attention matrix
   depth = query.shape[-1]
@@ -119,7 +117,7 @@ def dot_product_attention(query: Array,
 class MultiHeadDotProductAttention(Module):
   """Multi-head dot-product attention.
 
-    Args:
+    Attributes:
       num_heads: number of attention heads. Features (i.e. inputs_q.shape[-1])
         should be divisible by the number of heads.
       dtype: the dtype of the computation (default: float32)
@@ -127,7 +125,9 @@ class MultiHeadDotProductAttention(Module):
       out_features: dimension of the last projection
       broadcast_dropout: bool: use a broadcasted dropout along batch dims.
       dropout_rate: dropout rate
-      deterministic: bool, deterministic or not (to apply dropout)
+      deterministic: if false, the attention weight is masked randomly
+        using dropout, whereas if true, the attention weights
+        are deterministic.
       precision: numerical precision of the computation see `jax.lax.Precision`
         for details.
       kernel_init: initializer for the kernel of the Dense layers.
@@ -144,7 +144,7 @@ class MultiHeadDotProductAttention(Module):
   out_features: Optional[int] = None
   broadcast_dropout: bool = True
   dropout_rate: float = 0.
-  deterministic: bool = False
+  deterministic: Optional[bool] = None
   precision: Any = None
   kernel_init: Callable[[PRNGKey, Shape, Dtype], Array] = default_kernel_init
   bias_init: Callable[[PRNGKey, Shape, Dtype], Array] = zeros
@@ -156,7 +156,8 @@ class MultiHeadDotProductAttention(Module):
   def __call__(self,
                inputs_q: Array,
                inputs_kv: Array,
-               mask: Optional[Array] = None):
+               mask: Optional[Array] = None,
+               deterministic: Optional[bool] = None):
     """Applies multi-head dot product attention on the input data.
 
     Projects the inputs into multi-headed query, key, and value vectors,
@@ -169,10 +170,14 @@ class MultiHeadDotProductAttention(Module):
         `[batch_sizes..., length, features]`.
       mask: attention mask of shape
         `[batch_sizes..., num_heads, query_length, key/value_length]`.
+      deterministic: if false, the attention weight is masked randomly
+        using dropout, whereas if true, the attention weights
+        are deterministic.
 
     Returns:
       output of shape `[batch_sizes..., length, features]`.
     """
+    deterministic = merge_param('deterministic', self.deterministic, deterministic)
     features = self.out_features or inputs_q.shape[-1]
     qkv_features = self.qkv_features or inputs_q.shape[-1]
     assert qkv_features % self.num_heads == 0, (
@@ -240,7 +245,7 @@ class MultiHeadDotProductAttention(Module):
       attention_bias = None
 
     dropout_rng = None
-    if not self.deterministic and self.dropout_rate > 0.:
+    if not deterministic and self.dropout_rate > 0.:
       dropout_rng = self.make_rng('dropout')
 
     # apply attention
@@ -252,9 +257,9 @@ class MultiHeadDotProductAttention(Module):
         dropout_rng=dropout_rng,
         dropout_rate=self.dropout_rate,
         broadcast_dropout=self.broadcast_dropout,
-        deterministic=self.deterministic,
+        deterministic=deterministic,
         dtype=self.dtype,
-        precision=self.precision)
+        precision=self.precision)  # pytype: disable=wrong-keyword-args
 
     # back to the original inputs dimensions
     out = DenseGeneral(features=features,
@@ -272,8 +277,9 @@ class SelfAttention(MultiHeadDotProductAttention):
   """Self-attention special case of multi-head dot-product attention."""
 
   @compact
-  def __call__(self, inputs_q: Array, mask: Optional[Array] = None):
-    return super().__call__(inputs_q, inputs_q, mask)
+  def __call__(self, inputs_q: Array, mask: Optional[Array] = None,
+               deterministic: Optional[bool] = None):
+    return super().__call__(inputs_q, inputs_q, mask, deterministic=deterministic)
 
 
 # mask-making utility functions
@@ -281,7 +287,7 @@ class SelfAttention(MultiHeadDotProductAttention):
 
 def make_attention_mask(query_input: Array,
                         key_input: Array,
-                        pairwise_fn: Callable = jnp.multiply,
+                        pairwise_fn: Callable[..., Any] = jnp.multiply,
                         extra_batch_dims: int = 0,
                         dtype: Dtype = jnp.float32):
   """Mask-making helper for attention weights.
@@ -336,6 +342,7 @@ def combine_masks(*masks: Optional[Array], dtype: Dtype = jnp.float32):
 
   Args:
     *masks: set of attention mask arguments to combine, some can be None.
+    dtype: dtype for the returned mask.
 
   Returns:
     Combined mask, reduced by logical and, returns None if no masks given.
