@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +14,9 @@
 
 """Linear modules."""
 
-from collections.abc import Iterable  # pylint: disable=g-importing-member
 from dataclasses import field
 
-from typing import (Any, Callable, Sequence, Optional, Tuple, Union)
+from typing import (Any, Callable, Iterable, Optional, Tuple, Union)
 
 from flax.linen.module import Module, compact
 from flax.linen.initializers import lecun_normal, variance_scaling, zeros
@@ -28,7 +27,7 @@ import numpy as np
 
 
 PRNGKey = Any
-Shape = Tuple[int]
+Shape = Iterable[int]
 Dtype = Any  # this could be a real type?
 Array = Any
 
@@ -41,12 +40,20 @@ def _normalize_axes(axes, ndim):
   return tuple([ax if ax >= 0 else ndim + ax for ax in axes])
 
 
+def _canonicalize_tuple(x):
+  if isinstance(x, Iterable):
+    return tuple(x)
+  else:
+    return (x,)
+
+
 class DenseGeneral(Module):
   """A linear transformation with flexible axes.
 
-    Args:
-      features: tuple with numbers of output features.
-      axis: tuple with axes to apply the transformation on.
+    Attributes:
+      features: int or tuple with number of output features.
+      axis: int or tuple with axes to apply the transformation on. For instance,
+        (-2, -1) will apply the transformation to the last two axes.
       batch_dims: tuple with batch axes.
       use_bias: whether to add a bias to the output (default: True).
       dtype: the dtype of the computation (default: float32).
@@ -55,32 +62,15 @@ class DenseGeneral(Module):
       precision: numerical precision of the computation see `jax.lax.Precision`
         for details.
   """
-  features: int
-  axis: int = -1
-  batch_dims: Sequence[int] = ()
+  features: Union[int, Iterable[int]]
+  axis: Union[int, Iterable[int]] = -1
+  batch_dims: Iterable[int] = ()
   use_bias: bool = True
   dtype: Dtype = jnp.float32
   kernel_init: Callable[[PRNGKey, Shape, Dtype], Array] = default_kernel_init
   bias_init: Callable[[PRNGKey, Shape, Dtype], Array] = zeros
   precision: Any = None
 
-  def setup(self):
-    """Normalize hyperparameters."""
-    if not isinstance(self.features, Iterable):
-      self.features = (self.features,)
-    if not isinstance(self.axis, Iterable):
-      self.axis = (self.axis,)
-    if not isinstance(self.batch_dims, Iterable):
-      self.batch_dims = (self.batch_dims,)
-    self.features = tuple(self.features)
-    self.axis = tuple(self.axis)
-    self.batch_dims = tuple(self.batch_dims)
-    if self.batch_dims:
-      max_dim = np.max(self.batch_dims)
-      if set(self.batch_dims) != set(range(max_dim + 1)):
-        raise ValueError('batch_dims %s must be consecutive leading '
-                         'dimensions starting from 0.' % str(self.batch_dims))
-
   @compact
   def __call__(self, inputs: Array) -> Array:
     """Applies a linear transformation to the inputs along multiple dimensions.
@@ -91,13 +81,22 @@ class DenseGeneral(Module):
     Returns:
       The transformed input.
     """
+    features = _canonicalize_tuple(self.features)
+    axis = _canonicalize_tuple(self.axis)
+    batch_dims = _canonicalize_tuple(self.batch_dims)
+    if batch_dims:
+      max_dim = np.max(batch_dims)
+      if set(batch_dims) != set(range(max_dim + 1)):
+        raise ValueError('batch_dims %s must be consecutive leading '
+                         'dimensions starting from 0.' % str(batch_dims))
+
     inputs = jnp.asarray(inputs, self.dtype)
 
     ndim = inputs.ndim
-    n_batch_dims = len(self.batch_dims)
-    axis = _normalize_axes(self.axis, ndim)
-    batch_dims = _normalize_axes(self.batch_dims, ndim)
-    n_axis, n_features = len(axis), len(self.features)
+    n_batch_dims = len(batch_dims)
+    axis = _normalize_axes(axis, ndim)
+    batch_dims = _normalize_axes(batch_dims, ndim)
+    n_axis, n_features = len(axis), len(features)
 
     def kernel_init_wrap(rng, shape, dtype=jnp.float32):
       size_batch_dims = np.prod(shape[:n_batch_dims], dtype=np.int32)
@@ -108,7 +107,7 @@ class DenseGeneral(Module):
       return jnp.reshape(kernel, shape)
 
     batch_shape = tuple([inputs.shape[ax] for ax in batch_dims])
-    kernel_shape = tuple([inputs.shape[ax] for ax in axis]) + self.features
+    kernel_shape = tuple([inputs.shape[ax] for ax in axis]) + features
     kernel = self.param('kernel', kernel_init_wrap, batch_shape + kernel_shape)
     kernel = jnp.asarray(kernel, self.dtype)
 
@@ -126,7 +125,7 @@ class DenseGeneral(Module):
                                 for _ in range(size_batch_dims)], axis=0)
         return jnp.reshape(bias, shape)
 
-      bias = self.param('bias', bias_init_wrap, batch_shape + self.features)
+      bias = self.param('bias', bias_init_wrap, batch_shape + features)
 
       # Reshape bias for broadcast.
       expand_dims = sorted(
@@ -141,7 +140,7 @@ class DenseGeneral(Module):
 class Dense(Module):
   """A linear transformation applied over the last dimension of the input.
 
-  Args:
+  Attributes:
     features: the number of output features.
     use_bias: whether to add a bias to the output (default: True).
     dtype: the dtype of the computation (default: float32).
@@ -194,7 +193,7 @@ def _conv_dimension_numbers(input_shape):
 class Conv(Module):
   """Convolution Module wrapping lax.conv_general_dilated.
 
-  Args:
+  Attributes:
     features: number of convolution filters.
     kernel_size: shape of the convolutional kernel. For 1D convolution,
       the kernel size can be passed as an integer. For all other cases, it must
@@ -222,11 +221,11 @@ class Conv(Module):
     bias_init: initializer for the bias.
   """
   features: int
-  kernel_size: Union[int, Sequence[int]]
-  strides: Optional[Sequence[int]] = None
-  padding: Union[str, Sequence[Tuple[int, int]]] = 'SAME'
-  input_dilation: Optional[Sequence[int]] = None
-  kernel_dilation: Optional[Sequence[int]] = None
+  kernel_size: Union[int, Iterable[int]]
+  strides: Optional[Iterable[int]] = None
+  padding: Union[str, Iterable[Tuple[int, int]]] = 'SAME'
+  input_dilation: Optional[Iterable[int]] = None
+  kernel_dilation: Optional[Iterable[int]] = None
   feature_group_count: int = 1
   use_bias: bool = True
   dtype: Dtype = jnp.float32
@@ -257,8 +256,7 @@ class Conv(Module):
       is_single_input = True
       inputs = jnp.expand_dims(inputs, axis=0)
 
-    if self.strides is None:
-      self.strides = (1,) * (inputs.ndim - 2)
+    strides = self.strides or (1,) * (inputs.ndim - 2)
 
     in_features = inputs.shape[-1]
     assert in_features % self.feature_group_count == 0
@@ -271,7 +269,7 @@ class Conv(Module):
     y = lax.conv_general_dilated(
         inputs,
         kernel,
-        self.strides,
+        strides,
         self.padding,
         lhs_dilation=self.input_dilation,
         rhs_dilation=self.kernel_dilation,
@@ -289,9 +287,9 @@ class Conv(Module):
 
 
 class ConvTranspose(Module):
-  """Convolution Module wrapping lax.conv_general_dilated.
+  """Convolution Module wrapping lax.conv_transpose.
 
-  Args:
+  Attributes:
     features: number of convolution filters.
     kernel_size: shape of the convolutional kernel. For 1D convolution,
       the kernel size can be passed as an integer. For all other cases, it must
@@ -313,10 +311,10 @@ class ConvTranspose(Module):
     bias_init: initializer for the bias.
   """
   features: int
-  kernel_size: Union[int, Sequence[int]]
-  strides: Optional[Sequence[int]] = None
-  padding: Union[str, Sequence[Tuple[int, int]]] = 'SAME'
-  kernel_dilation: Optional[Sequence[int]] = None
+  kernel_size: Union[int, Iterable[int]]
+  strides: Optional[Iterable[int]] = None
+  padding: Union[str, Iterable[Tuple[int, int]]] = 'SAME'
+  kernel_dilation: Optional[Iterable[int]] = None
   use_bias: bool = True
   dtype: Dtype = jnp.float32
   precision: Any = None
@@ -369,14 +367,15 @@ class ConvTranspose(Module):
     return y
 
 
-default_embed_init = variance_scaling(1.0, 'fan_in', 'normal',
-                                                   out_axis=0)
+default_embed_init = variance_scaling(1.0, 'fan_in', 'normal', out_axis=0)
+
 
 class Embed(Module):
   """Embedding Module.
+
   A parameterized function from integers [0, n) to d-dimensional vectors.
 
-  Args:
+  Attributes:
     num_embeddings: number of embeddings.
     features: number of feature dimensions for each embedding.
     dtype: the dtype of the embedding vectors (default: float32).
