@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@ from jax import lax
 from jax.nn import initializers
 import jax.numpy as jnp
 
-from flax.linen.module import Module, compact
+from flax.linen.module import Module, compact, merge_param
 
 
 PRNGKey = Any
@@ -38,16 +38,16 @@ def _absolute_dims(rank, dims):
 class BatchNorm(Module):
   """BatchNorm Module.
 
-  Args:
-    use_running_average: if true, the statistics stored in batch_stats
+  Attributes:
+    use_running_average: if True, the statistics stored in batch_stats
       will be used instead of computing the batch statistics on the input.
     axis: the feature or non-batch axis of the input.
     momentum: decay rate for the exponential moving average of
       the batch statistics.
     epsilon: a small float added to variance to avoid dividing by zero.
     dtype: the dtype of the computation (default: float32).
-    bias:  if True, bias (beta) is added.
-    scale: if True, multiply by scale (gamma).
+    use_bias:  if True, bias (beta) is added.
+    use_scale: if True, multiply by scale (gamma).
       When the next layer is linear (also e.g. nn.relu), this can be disabled
       since the scaling will be done by the next layer.
     bias_init: initializer for bias, by default, zero.
@@ -60,7 +60,7 @@ class BatchNorm(Module):
       the examples on the first two and last two devices. See `jax.lax.psum`
       for more details.
   """
-  use_running_average: bool = False
+  use_running_average: Optional[bool] = None
   axis: int = -1
   momentum: float = 0.99
   epsilon: float = 1e-5
@@ -73,15 +73,19 @@ class BatchNorm(Module):
   axis_index_groups: Any = None
 
   @compact
-  def __call__(self, x):
+  def __call__(self, x, use_running_average: Optional[bool] = None):
     """Normalizes the input using batch statistics.
 
     Args:
       x: the input to be normalized.
+      use_running_average: if true, the statistics stored in batch_stats
+        will be used instead of computing the batch statistics on the input.
 
     Returns:
       Normalized inputs (the same shape as inputs).
     """
+    use_running_average = merge_param(
+        'use_running_average', self.use_running_average, use_running_average)
     x = jnp.asarray(x, jnp.float32)
     axis = self.axis if isinstance(self.axis, tuple) else (self.axis,)
     axis = _absolute_dims(x.ndim, axis)
@@ -99,7 +103,7 @@ class BatchNorm(Module):
                            lambda s: jnp.ones(s, jnp.float32),
                            reduced_feature_shape)
 
-    if self.use_running_average:
+    if use_running_average:
       mean, var = ra_mean.value, ra_var.value
     else:
       mean = jnp.mean(x, axis=reduction_axis, keepdims=False)
@@ -142,7 +146,7 @@ class LayerNorm(Module):
   i.e. applies a transformation that maintains the mean activation within
   each example close to 0 and the activation standard deviation close to 1.
 
-  Args:
+  Attributes:
     epsilon: A small float added to variance to avoid dividing by zero.
     dtype: the dtype of the computation (default: float32).
     use_bias:  If True, bias (beta) is added.
@@ -189,6 +193,7 @@ class LayerNorm(Module):
 
 class GroupNorm(Module):
   """Group normalization (arxiv.org/abs/1803.08494).
+  
     This op is similar to batch normalization, but statistics are shared across
     equally-sized groups of channels and not shared across batch dimension.
     Thus, group normalization does not depend on the batch composition and does
@@ -196,14 +201,14 @@ class GroupNorm(Module):
     The user should either specify the total number of channel groups or the
     number of channels per group.
 
-    Args:
+    Attributes:
       num_groups: the total number of channel groups. The default value of 32 is
         proposed by the original group normalization paper.
       group_size: the number of channels in a group.
       epsilon: A small float added to variance to avoid dividing by zero.
       dtype: the dtype of the computation (default: float32).
-      bias:  If True, bias (beta) is added.
-      scale: If True, multiply by scale (gamma). When the next layer is linear
+      use_bias:  If True, bias (beta) is added.
+      use_scale: If True, multiply by scale (gamma). When the next layer is linear
         (also e.g. nn.relu), this can be disabled since the scaling will be done
         by the next layer.
       bias_init: Initializer for bias, by default, zero.
