@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,12 +26,42 @@ with warnings.catch_warnings():
   else:
     mpl.use('Agg')
 # pylint: disable=g-import-not-at-top
-import numpy as onp
+import numpy as np
 
 from tensorboard.plugins.hparams import api as hparams_api
 import tensorflow.compat.v2 as tf
 
 
+def _flatten_dict(input_dict, parent_key='', sep='.'):
+  """Flatten and simplify dict such that it can be used by hparams.
+
+  Args:
+    input_dict: Input dict, e.g., from ConfigDict.
+    parent_key: String used in recursion.
+    sep: String used to separate parent and child keys.
+
+  Returns:
+   Flattened dict.
+  """
+  items = []
+  for k, v in input_dict.items():
+    new_key = parent_key + sep + k if parent_key else k
+
+    # Take special care of things hparams cannot handle.
+    if v is None:
+      v = 'None'
+    elif isinstance(v, list):
+      v = str(v)
+    elif isinstance(v, tuple):
+      v = str(v)
+    elif isinstance(v, dict):
+      # Recursively flatten the dict.
+      items.extend(_flatten_dict(v, new_key, sep=sep).items())
+    else:
+      items.append((new_key, v))
+  return dict(items)
+
+
 class SummaryWriter(object):
   """Saves data in event and summary protos for tensorboard."""
 
@@ -66,12 +96,12 @@ class SummaryWriter(object):
       value: int/float: number to log
       step: int: training step
     """
-    value = float(onp.array(value))
+    value = float(np.array(value))
     with self._event_writer.as_default():
       tf.summary.scalar(name=tag, data=value, step=step)
 
   def image(self, tag, image, step):
-    """Saves RGB image summary from onp.ndarray [H,W], [H,W,1], or [H,W,3].
+    """Saves RGB image summary from np.ndarray [H,W], [H,W,1], or [H,W,3].
 
     Args:
       tag: str: label for this data
@@ -80,14 +110,14 @@ class SummaryWriter(object):
         Floating point values should be in range [0, 1).
       step: int: training step
     """
-    image = onp.array(image)
-    if len(onp.shape(image)) == 2:
-      image = image[:, :, onp.newaxis]
-    if onp.shape(image)[-1] == 1:
-      image = onp.repeat(image, 3, axis=-1)
+    image = np.array(image)
+    if len(np.shape(image)) == 2:
+      image = image[:, :, np.newaxis]
+    if np.shape(image)[-1] == 1:
+      image = np.repeat(image, 3, axis=-1)
     # tf.summary.image expects image to have shape [k, h, w, c] where,
     # k = number of samples, h = height, w = width, c = number of channels.
-    image = image[onp.newaxis, :, :, :]
+    image = image[np.newaxis, :, :, :]
 
     # Convert to tensor value as tf.summary.image expects data to be a tensor.
     image = tf.convert_to_tensor(image)
@@ -110,7 +140,7 @@ class SummaryWriter(object):
     """
     # tf.summary.audio expects the audio data to have floating values in
     # [-1.0, 1.0].
-    audiodata = onp.clip(onp.array(audiodata), -1, 1)
+    audiodata = np.clip(np.array(audiodata), -1, 1)
 
     # Convert to tensor value as tf.summary.audio expects data to be a tensor.
     audio = tf.convert_to_tensor(audiodata, dtype=tf.float32)
@@ -128,8 +158,8 @@ class SummaryWriter(object):
       step: int: training step
       bins: number of bins in histogram
     """
-    values = onp.array(values)
-    values = onp.reshape(values, -1)
+    values = np.array(values)
+    values = np.reshape(values, -1)
     with self._event_writer.as_default():
       tf.summary.histogram(name=tag, data=values, step=step, buckets=bins)
 
@@ -172,5 +202,6 @@ class SummaryWriter(object):
     Args:
       hparams: Flat mapping from hyper parameter name to value.
     """
+
     with self._event_writer.as_default():
-      hparams_api.hparams(hparams=hparams)
+      hparams_api.hparams(hparams=_flatten_dict(hparams))
