@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@ from jax import lax
 from jax import random
 import jax.numpy as jnp
 
-import numpy as onp
+import numpy as np
 
 
 def dot_product_attention(query,
@@ -45,7 +45,11 @@ def dot_product_attention(query,
                           dropout_rate=0.,
                           deterministic=False,
                           precision=None):
-  """Computes dot-product attention given query, key, and value.
+  """DEPRECATION WARNING:
+ "The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Computes dot-product attention given query, key, and value.
 
   This is the core function for applying attention based on
   https://arxiv.org/abs/1706.03762. It calculates the attention weights given
@@ -91,7 +95,7 @@ def dot_product_attention(query,
   depth = query.shape[-1]
   n = key.ndim
   # batch_dims is  <bs, <non-attention dims>, num_heads>
-  batch_dims = tuple(onp.delete(range(n), axis + (n - 1,)))
+  batch_dims = tuple(np.delete(range(n), axis + (n - 1,)))
   # q & k -> (bs, <non-attention dims>, num_heads, <attention dims>, channels)
   qk_perm = batch_dims + axis + (n - 1,)
   key = key.transpose(qk_perm)
@@ -154,9 +158,9 @@ def _invert_perm(perm):
 
 @struct.dataclass
 class _CacheEntry:
-  key: onp.ndarray
-  value: onp.ndarray
-  i: onp.ndarray
+  key: np.ndarray
+  value: np.ndarray
+  i: np.ndarray
 
 
 def scan_in_dim(*args, **kwargs):
@@ -166,7 +170,10 @@ def scan_in_dim(*args, **kwargs):
 
 
 class Cache(Collection):
-  """Collect intermediate activations for efficient autoregressive decoding."""
+  """The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Collect intermediate activations for efficient autoregressive decoding."""
 
   def initialize_cache(self, shape, dtype=None):
     """Initialize the cache for the given input shape.
@@ -197,7 +204,10 @@ jax.tree_util.register_pytree_node(
 
 
 class MultiHeadDotProductAttention(Module):
-  """Multi-head dot-product attention."""
+  """The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Multi-head dot-product attention."""
 
   def apply(self,
             inputs_q,
@@ -304,7 +314,7 @@ class MultiHeadDotProductAttention(Module):
     if cache:
       assert isinstance(cache, Cache), 'cache must be an instance of Cache'
       if self.is_initializing():
-        cache.store(onp.array((key.ndim,) + key.shape[-2:], dtype=onp.int32))
+        cache.store(np.array((key.ndim,) + key.shape[-2:], dtype=np.int32))
       else:
         cache_entry = cache.retrieve(None)
         expected_shape = list(cache_entry.key.shape[:-2])
@@ -322,7 +332,7 @@ class MultiHeadDotProductAttention(Module):
         cshape = cache_entry.key.shape
         indices = [0] * len(cshape)
         i = cache_entry.i
-        attn_size = onp.prod(onp.take(cshape, attention_axis))
+        attn_size = np.prod(np.take(cshape, attention_axis))
         for attn_dim in attention_axis:
           attn_size //= cshape[attn_dim]
           indices[attn_dim] = i // attn_size
@@ -342,8 +352,8 @@ class MultiHeadDotProductAttention(Module):
     if causal_mask:
       if cache and not self.is_initializing():
         bias_pre_shape = (1,) * (key.ndim - 1)
-        attn_shape = tuple(onp.take(key.shape, attention_axis))
-        attn_size = onp.prod(attn_shape)
+        attn_shape = tuple(np.take(key.shape, attention_axis))
+        attn_size = np.prod(attn_shape)
         ii = jnp.arange(attn_size, dtype=jnp.uint32)
         mask = ii < cache_entry.i
         mask_components.append(mask.reshape(bias_pre_shape + attn_shape))
@@ -437,7 +447,10 @@ def make_padding_mask(padding_mask_query,
                       key_shape,
                       attention_axis=None,
                       segmentation_mask=False):
-  """Makes padding mask for attention weights.
+  """The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Makes padding mask for attention weights.
 
   In case of 1d inputs (i.e., `[bs, len, features]`, the attention weights will
   be `[bs, len, len]` and this function makes a square matrix [len, len].
@@ -474,7 +487,7 @@ def make_padding_mask(padding_mask_query,
 
   padding_mask_query = padding_mask_query[..., None]
   padding_mask_key = padding_mask_key[..., None]
-  perm = (0,) + tuple(onp.flip(onp.arange(padding_mask_key.ndim)))[:-1]
+  perm = (0,) + tuple(np.flip(np.arange(padding_mask_key.ndim)))[:-1]
   if segmentation_mask:
     mask = jnp.equal(padding_mask_query, padding_mask_key.transpose(perm))
   else:
@@ -486,7 +499,10 @@ def make_padding_mask(padding_mask_query,
 
 
 def _make_causal_mask(key, attention_axis=None, self_mask=False):
-  """Makes a causal mask, to be used for masking out the future for attention.
+  """The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Makes a causal mask, to be used for masking out the future for attention.
 
   In case of 1d inputs (i.e., `[bs, len, features]`, the attention weights will
   be `[bs, len, len]` and this function makes a square matrix [len, len] with
