@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""NN base modules for JAX."""
+"""DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  NN base modules for JAX."""
 
 from typing import Type
 
@@ -47,7 +51,11 @@ def _track_outputs(x):
 
 
 class _ModuleFrame:
-  """A ModuleFrame the context needed to initialize (init) or apply a Module.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A ModuleFrame the context needed to initialize (init) or apply a Module.
 
   In particular, `self.params` is a dictionary where parameters are
   stored (during module init) and read from (during module application).
@@ -131,7 +139,11 @@ class _ModuleFrame:
 
 
 def module_method(fn):
-  """Decorates a function as a module method.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Decorates a function as a module method.
 
   The `module_method` allows modules to have multiple methods that make use of
   the modules parameters.
@@ -198,7 +210,11 @@ MODULE_CLASSMETHODS = [
 
 
 class _ModuleMeta(abc.ABCMeta):
-  """A meta class for automatically setting the doc of Modules."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A meta class for automatically setting the doc of Modules."""
 
   def __init__(cls, name, bases, attrs):
     super(_ModuleMeta, cls).__init__(name, bases, attrs)
@@ -242,12 +258,16 @@ def _fold_in_str(rng, data):
   m = hashlib.sha1()
   m.update(data.encode('utf-8'))
   d = m.digest()
-  hash_int = int.from_bytes(d[:4], byteorder='big')
+  hash_int = int.from_bytes(d[:4], byteorder='big', signed=True)
   return random.fold_in(rng, hash_int)
 
 
 class Module(metaclass=_ModuleMeta):
-  """Functional modules."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Functional modules."""
 
   def __new__(cls, *args, name=None, **kwargs):
     if not _module_stack:
@@ -693,7 +713,11 @@ class Module(metaclass=_ModuleMeta):
 
 
 def module(fun):
-  """Convert a function into the apply method of a new Module.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Convert a function into the apply method of a new Module.
 
   This is convenient shortcut for writing higher level modules that don't need
   access to `self` for creating parameters directly.
@@ -728,7 +752,11 @@ def module(fun):
 
 # TODO(flax-dev) consider removing this...
 class TransparentModule(Module):
-  """A transparent module.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A transparent module.
 
   A transparent module can only have one parameter named '0'.
   """
@@ -753,7 +781,11 @@ class TransparentModule(Module):
 
 
 class TruncatedModule(TransparentModule):
-  """Wraps a Module and returns the requested intermediate outputs instead.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Wraps a Module and returns the requested intermediate outputs instead.
 
   Check `Model.truncate_at` for a simple API to get the intermediate outputs of
   an existing Model.
@@ -785,7 +817,11 @@ class TruncatedModule(TransparentModule):
 
 @contextlib.contextmanager
 def capture_module_outputs():
-  """A context manager that captures all model outputs.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A context manager that captures all model outputs.
 
   Yields:
     A `flax.nn.Collection` containing all module outputs.
@@ -796,7 +832,11 @@ def capture_module_outputs():
 
 
 class ModuleState():
-  """Tracks a state variable.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Tracks a state variable.
 
   ModuleState instances should not be created directly. See `Module.state` on
   how to create state variables inside modules.
@@ -831,7 +871,11 @@ class ModuleState():
 
 @contextlib.contextmanager
 def stateful(state=None, mutable=True):
-  """A context manager for stateful computations.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A context manager for stateful computations.
 
   Module's that use the `Module.state` by default store state inside the
   `Collection` specified by the (innermost) `nn.stateful` context manager.
@@ -882,7 +926,11 @@ def stateful(state=None, mutable=True):
 
 
 def is_stateful():
-  """Returns true if a stateful scope is currently active (see `flax.nn.stateful`)."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Returns true if a stateful scope is currently active (see `flax.nn.stateful`)."""
   return bool(_state_stack)
 
 
@@ -902,7 +950,11 @@ def _top_frame(call_name):
 
 @struct.dataclass
 class Model:
-  """A Model contains the model parameters, state and definition."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A Model contains the model parameters, state and definition."""
 
   module: Type[Module] = struct.field(pytree_node=False)
   params: Any = struct.field(pytree_node=True)
@@ -942,7 +994,11 @@ class Model:
 
 
 class Collection:
-  """A collection of tensors useful for tracking state.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  A collection of tensors useful for tracking state.
 
   A Collection can be used to associate data with the application of a Module.
   For example, a collection can be used to collect activations across modules.
