@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,15 @@
 from jax import lax
 import jax.numpy as jnp
 
-import numpy as onp
+import numpy as np
 
 
 def pool(inputs, init, reduce_fn, window_shape, strides, padding):
-  """Helper function to define pooling functions.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Helper function to define pooling functions.
 
   Pooling functions are implemented using the ReduceWindow XLA op.
   NOTE: Be aware that pooling is not generally differentiable.
@@ -56,7 +60,11 @@ def pool(inputs, init, reduce_fn, window_shape, strides, padding):
 
 
 def avg_pool(inputs, window_shape, strides=None, padding="VALID"):
-  """Pools the input by taking the average over a window.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Pools the input by taking the average over a window.
 
   Args:
     inputs: input data with dimensions (batch, window dims..., features).
@@ -70,12 +78,16 @@ def avg_pool(inputs, window_shape, strides=None, padding="VALID"):
     The average for each window slice.
   """
   y = pool(inputs, 0., lax.add, window_shape, strides, padding)
-  y = y / onp.prod(window_shape)
+  y = y / np.prod(window_shape)
   return y
 
 
 def max_pool(inputs, window_shape, strides=None, padding="VALID"):
-  """Pools the input by taking the maximum of a window slice.
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Pools the input by taking the maximum of a window slice.
 
   Args:
     inputs: input data with dimensions (batch, window dims..., features).
