@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""NN base modules for JAX."""
+"""DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  NN base modules for JAX."""
 
 import contextlib
 import threading
@@ -20,7 +24,11 @@ import jax
 
 
 class CallStack(object):
-  """Utility for tracking data across a call stack."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Utility for tracking data across a call stack."""
 
   def __init__(self):
     self._stack = threading.local()
@@ -52,7 +60,11 @@ class CallStack(object):
 
 
 def classproperty(f):
-  """decorator that registers a function as a read-only property of the class."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  decorator that registers a function as a read-only property of the class."""
 
   class _ClassProperty:
 
@@ -65,21 +77,33 @@ def classproperty(f):
 
 
 def _masters():
-  """Returns a list of currently active Jax tracers."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Returns a list of currently active Jax tracers."""
   # TODO(jheek): consider re-introducing the tracer check
   # for now we pretent there are never any tracers
   return ()
 
 
 def _trace_level(master):
-  """Returns the level of the trace of -infinity if it is None."""
+  """DEPRECATION WARNING:
+ The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Returns the level of the trace of -infinity if it is None."""
   if master:
     return master.level
   return float('-inf')
 
 
 def _current_trace():
-  """Returns the innermost Jax tracer."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Returns the innermost Jax tracer."""
   tracers = _masters()
   if tracers:
     return tracers[-1]
@@ -87,7 +111,11 @@ def _current_trace():
 
 
 def _level_of_value(xs):
-  """Returns the tracer level associated with a value if any."""
+  """DEPRECATION WARNING:
+  The `flax.nn` module is Deprecated, use `flax.linen` instead. 
+  Learn more and find an upgrade guide at 
+  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  Returns the tracer level associated with a value if any."""
   xs = jax.tree_leaves(xs)
   max_level = float('-inf')
   # TODO(jheek): consider re-introducing the tracer check
