@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 import jax.numpy as jnp
-import numpy as onp
+import numpy as np
 from .. import struct
 from .base import OptimizerDef
 
@@ -30,7 +30,7 @@ class _AdagradHyperParams:
 class _AdagradParamState:
   """Adagrad parameter state"""
 
-  G: onp.ndarray
+  G: np.ndarray
 
 
 class Adagrad(OptimizerDef):
